import { AssistantContent, ToolContent, ToolResultPart, UserContent, Message as AiMessage, CoreMessage } from 'ai';
import { MastraBase } from '../base';
export type AiMessageType = AiMessage;
export type MessageType = {
    id: string;
    content: UserContent | AssistantContent | ToolContent;
    role: 'user' | 'assistant' | 'tool';
    createdAt: Date;
    threadId: string;
    toolCallIds?: string[];
    toolCallArgs?: Record<string, unknown>[];
    toolNames?: string[];
    type: 'text' | 'tool-call' | 'tool-result';
};
export type ThreadType = {
    id: string;
    title?: string;
    resourceid: string;
    createdAt: Date;
    updatedAt: Date;
    metadata?: Record<string, unknown>;
};
export type MessageResponse<T extends 'raw' | 'core_message'> = {
    raw: MessageType[];
    core_message: CoreMessage[];
}[T];
/**
 * Abstract Memory class that defines the interface for storing and retrieving
 * conversation threads and messages.
 */
export declare abstract class MastraMemory extends MastraBase {
    MAX_CONTEXT_TOKENS?: number;
    constructor();
    estimateTokens(text: string): number;
    parseMessages(messages: MessageType[]): MessageType[];
    convertToUIMessages(messages: MessageType[]): AiMessageType[];
    /**
     * Retrieves a specific thread by its ID
     * @param threadId - The unique identifier of the thread
     * @returns Promise resolving to the thread or null if not found
     */
    abstract getThreadById({ threadId }: {
        threadId: string;
    }): Promise<ThreadType | null>;
    abstract getThreadsByResourceId({ resourceid }: {
        resourceid: string;
    }): Promise<ThreadType[]>;
    /**
     * Saves or updates a thread
     * @param thread - The thread data to save
     * @returns Promise resolving to the saved thread
     */
    abstract saveThread({ thread }: {
        thread: ThreadType;
    }): Promise<ThreadType>;
    /**
     * Saves messages to a thread
     * @param messages - Array of messages to save
     * @returns Promise resolving to the saved messages
     */
    abstract saveMessages({ messages }: {
        messages: MessageType[];
    }): Promise<MessageType[]>;
    /**
     * Retrieves all messages for a specific thread
     * @param threadId - The unique identifier of the thread
     * @returns Promise resolving to array of messages and uiMessages
     */
    abstract getMessages({ threadId, }: {
        threadId: string;
    }): Promise<{
        messages: MessageType[];
        uiMessages: AiMessageType[];
    }>;
    /**
     * Retrieves all messages for a specific thread within a context window
     * @param threadId - The unique identifier of the thread
     * @param startDate - Optional start date to filter the context window
     * @param endDate - Optional end date to filter the context window
     * @returns Promise resolving to an array of messages
     */
    abstract getContextWindow<T extends 'raw' | 'core_message'>({ threadId, startDate, endDate, format, }: {
        threadId: string;
        startDate?: Date;
        endDate?: Date;
        format?: T;
    }): Promise<MessageResponse<T>>;
    /**
     * Retrieves cached tool result for a specific arg in a thread
     * @param threadId - The unique identifier of the thread
     * @param toolArgs - The tool arguments to retrieve the cached result for
     * @param toolName - The name of the tool that was called
     * @returns Promise resolving to the cached tool result or null if not found
     */
    abstract getToolResult({ threadId, toolArgs, toolName, }: {
        threadId: string;
        toolArgs: Record<string, unknown>;
        toolName: string;
    }): Promise<ToolResultPart['result'] | null>;
    /**
     * Checks if an un-expired tool call arg exists in a thread
     * @param hashedArgs - The hashed tool call information (args, threadId, toolName) to check for
     * @returns Promise resolving to true if the un-expired tool call arg exists, false otherwise
     */
    abstract validateToolCallArgs({ hashedArgs }: {
        hashedArgs: string;
    }): Promise<boolean>;
    /**
     * Helper method to create a new thread
     * @param title - Optional title for the thread
     * @param metadata - Optional metadata for the thread
     * @returns Promise resolving to the created thread
     */
    createThread({ threadId, resourceid, title, metadata, }: {
        resourceid: string;
        threadId?: string;
        title?: string;
        metadata?: Record<string, unknown>;
    }): Promise<ThreadType>;
    /**
     * Helper method to delete a thread
     * @param threadId - the id of the thread to delete
     */
    abstract deleteThread(threadId: string): Promise<void>;
    /**
     * Helper method to add a single message to a thread
     * @param threadId - The thread to add the message to
     * @param content - The message content
     * @param role - The role of the message sender
     * @param type - The type of the message
     * @param toolNames - Optional array of tool names that were called
     * @param toolCallArgs - Optional array of tool call arguments
     * @param toolCallIds - Optional array of tool call ids
     * @returns Promise resolving to the saved message
     */
    addMessage({ threadId, content, role, type, toolNames, toolCallArgs, toolCallIds, }: {
        threadId: string;
        content: UserContent | AssistantContent;
        role: 'user' | 'assistant';
        type: 'text' | 'tool-call' | 'tool-result';
        toolNames?: string[];
        toolCallArgs?: Record<string, unknown>[];
        toolCallIds?: string[];
    }): Promise<MessageType>;
    /**
     * Generates a unique identifier
     * @returns A unique string ID
     */
    generateId(): string;
}
//# sourceMappingURL=index.d.ts.map