import { Snapshot } from 'xstate';
import { z } from 'zod';
import { IAction, MastraPrimitives } from '../action';
import { MastraBase } from '../base';
import { WorkflowSnapshot } from './snapshot';
import { Step } from './step';
import { RetryConfig, StepConfig, StepGraph, StepResult, StepVariableType } from './types';
interface WorkflowResultReturn<T extends z.ZodType<any>> {
    runId: string;
    start: (props?: {
        triggerData?: z.infer<T>;
    } | undefined) => Promise<{
        triggerData?: z.infer<T>;
        results: Record<string, StepResult<any>>;
        runId: string;
    }>;
}
interface WorkflowRunState {
    value: Record<string, string>;
    context: {
        stepResults: Record<string, {
            status: 'success' | 'failed' | 'suspended' | 'waiting';
            payload?: any;
            error?: string;
        }>;
        triggerData: Record<string, any>;
        attempts: Record<string, number>;
    };
    activePaths: Array<{
        stepPath: string[];
        stepId: string;
        status: string;
    }>;
    runId: string;
    timestamp: number;
}
export declare class Workflow<TSteps extends Step<any, any, any>[] = any, TTriggerSchema extends z.ZodType<any> = any> extends MastraBase {
    #private;
    name: string;
    snapshot?: WorkflowSnapshot;
    triggerSchema?: TTriggerSchema;
    /**
     * Creates a new Workflow instance
     * @param name - Identifier for the workflow (not necessarily unique)
     * @param logger - Optional logger instance
     */
    constructor({ name, triggerSchema, retryConfig, mastra, }: {
        name: string;
        triggerSchema?: TTriggerSchema;
        retryConfig?: RetryConfig;
        mastra?: MastraPrimitives;
    });
    /**
     * Initializes the XState machine for the workflow
     *
     * Registers the machine's types, actions, actors, initial context, entry actions, initial state, and states
     * @returns The initialized machine
     */
    private initializeMachine;
    step<TStep extends IAction<any, any, any, any>, CondStep extends StepVariableType<any, any, any, any>, VarStep extends StepVariableType<any, any, any, any>>(step: TStep, config?: StepConfig<TStep, CondStep, VarStep, TTriggerSchema>): this;
    then<TStep extends IAction<any, any, any, any>, CondStep extends StepVariableType<any, any, any, any>, VarStep extends StepVariableType<any, any, any, any>>(step: TStep, config?: StepConfig<TStep, CondStep, VarStep, TTriggerSchema>): this;
    after<TStep extends IAction<any, any, any, any>>(step: TStep): Omit<typeof this, "then" | "after">;
    /**
     * Executes the workflow with the given trigger data
     * @param triggerData - Initial data to start the workflow with
     * @returns Promise resolving to workflow results or rejecting with error
     * @throws Error if trigger schema validation fails
     */
    createRun(): WorkflowResultReturn<TTriggerSchema>;
    execute({ triggerData, snapshot, runId, stepId, }?: {
        stepId?: string;
        triggerData?: z.infer<TTriggerSchema>;
        runId?: string;
        snapshot?: Snapshot<any>;
    }): Promise<{
        triggerData?: z.infer<TTriggerSchema>;
        results: Record<string, StepResult<any>>;
        runId: string;
    }>;
    /**
     * Rebuilds the machine with the current steps configuration and validates the workflow
     *
     * This is the last step of a workflow builder method chain
     * @throws Error if validation fails
     *
     * @returns this instance for method chaining
     */
    commit(): this;
    getState(runId: string): Promise<WorkflowRunState | null>;
    watch(runId: string, options?: {
        onTransition?: (state: WorkflowRunState) => void | Promise<void>;
        pollInterval?: number;
    }): Promise<WorkflowRunState>;
    resume({ runId, stepId, context: resumeContext, }: {
        runId: string;
        stepId: string;
        context?: Record<string, any>;
    }): Promise<{
        triggerData?: z.infer<TTriggerSchema>;
        results: Record<string, StepResult<any>>;
        runId: string;
    }>;
    __registerPrimitives(p: MastraPrimitives): void;
    get stepGraph(): StepGraph;
    get stepSubscriberGraph(): Record<string, StepGraph>;
    get steps(): Record<string, IAction<any, any, any, any>>;
}
export {};
//# sourceMappingURL=workflow.d.ts.map